package timer;

import java.io.*;
import java.net.*;

/**
 * <p>Title: MultiClientTimerThread</p>
 *
 * <p>Description: Thread for each of the clients accepted for the server.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Adrin Riesco
 * @version 1.0
 */
public class MultiClientTimerThread extends Thread {

    /**
     * Socket through where the messages are sent.
     */
    private Socket socket = null;

    /**
     * Attribute used to write in the socket.
     */
    private PrintWriter out;

    /**
     * Attribute used to read from the socket.
     */
    private BufferedReader in;

    /**
     * Constructor, initializes the attributes.
     * @param socket Socket used in the communication.
     */
    public MultiClientTimerThread(Socket socket) {
        super("MultiClientTimerThread");
        this.socket = socket;
        try{
            out = new PrintWriter(socket.getOutputStream(), true);
            in = new BufferedReader(new InputStreamReader(
                    socket.getInputStream()));
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Thread execution.
     */
    public void run(){
        try {
            String inputLine;
            while ((inputLine = in.readLine()) != null &&
                    inputLine.indexOf("#") != -1) {
                // We delete the #
                inputLine = inputLine.substring(0, inputLine.length() - 1);
                cases(inputLine);
            }
            close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Function that executes the different cases obtained from the input.
     * @param input String obtained from the socket.
     */
    private void cases(String input){
        int caso = parse(input);
        switch (caso) {
        case 0 :
            wait(input);
            break;
        default :
            break;
        }
    }

    /**
     * Function that closes the attributes.
     */
    private void close(){
        try{
            out.close();
            in.close();
            socket.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Function that parses the input in order to decide what kind of
     * instruction it is.
     * @param input String obtained from the socket.
     * @return Number of instruction.
     */
    private int parse(String input) {
        int resultado = -1;
        if (input. startsWith("wait(")){
            resultado = 0;
        }
        return resultado;
    }

    /**
     * Function that extracts the milliseconds from the input in an
     * instruction wait(N).
     * @param input String obtained from the socket.
     * @return Number of milliseconds indicated in the message.
     */
    private int getWait(String input) {
        // We delete "wait(" from the input.
        String strTime = input.substring(5);
        // We delete the ")" from the input.
        strTime = strTime.substring(0, strTime.length()-1);
        Integer i = new Integer(strTime);
        int time = i.intValue();
        return time;
    }

    /**
     * Function that extracts the number of milliseconds that must wait from
     * the input and then waits.
     * @param input String received through the socket.
     */
    private void wait(String input){
        try{
            int time = getWait(input);
            sleep(time);
            out.println("tick#");
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

}
